import { httpApp } from "@/plugins/axiosApp";
import router from "@/router";
import { AxiosResponse } from "axios";

// 下载文件
export async function downloadFileURL(
  url: string,
  data: Record<string, any> = {}
) {
  return httpApp
    .get(url, { responseType: "blob", params: data })
    .then((res) => {
      return responseToFile(res);
    })
    .catch((err) => console.log(err));
}

export async function responseToFile(response: AxiosResponse) {
  return new Promise((resolve) => {
    const { data, headers } = response;
    const fileName = headers["content-disposition"].replace(
      /\w+;\s*filename=(.*)/,
      "$1"
    );
    //Here, when the JSON file is returned, the JSON.stringify Processing, other types of files do not need to be processed
    //const blob = new Blob([JSON.stringify(data)], ...)
    const blob = new Blob([data], { type: headers["content-type"] });
    const dom = document.createElement("a");
    const url = window.URL.createObjectURL(blob);
    dom.href = url;
    dom.download = decodeURI(fileName);
    dom.style.display = "none";
    document.body.appendChild(dom);
    dom.click();
    dom.parentNode?.removeChild(dom);
    window.URL.revokeObjectURL(url);
    resolve(true);
  });
}

export function downloadByLink(url: string) {
  const dom = document.createElement("a");
  dom.href = url;
  let filename = url.split("/").slice(-1)[0];
  dom.download = decodeURI(filename);
  dom.style.display = "none";
  document.body.appendChild(dom);
  dom.click();
  dom.parentNode?.removeChild(dom);
}

// 返回
export function goBack() {
  router.back();
}

/**
 * 将目标对象中有的属性值与源对象中的属性值合并
 * @param {Object} target 目标对象
 * @param {Object} sources 源对象
 */
export function objAssign<T extends object>(
  target: object,
  sources: object
): T {
  const targ = { ...target };
  Object.keys(targ).forEach((k) => {
    targ[k] = Object.prototype.hasOwnProperty.call(sources, k)
      ? sources[k]
      : targ[k];
  });

  return targ;
}
/**
 * 使用目标对象中有的属性值修改源对象中的属性值
 * @param {Object} target 目标对象
 * @param {Object} sources 源对象
 */
export function objModifyAssign(target: object, sources: object) {
  Object.keys(target).forEach((k) => {
    if (Object.prototype.hasOwnProperty.call(sources, k)) {
      target[k] = sources[k];
    }
  });
}
